// src/backend/dynamic_request.cc - Copyright 2005, 2006, University
//               of Padova, dept. of Pure and Applied
//               Mathematics
//
// This file is part of SGPEMv2.
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// SGPEMv2 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with SGPEMv2; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include "dynamic_request.hh"
#include "static_request.hh"
#include "dynamic_sub_request.hh"
#include "dynamic_thread.hh"
#include "serialize_visitor.hh"

#include "smartp.tcc"

#include <algorithm>
#include <functional>
#include <cassert>

using namespace sgpem;
using namespace std;

DynamicRequest::DynamicRequest(StaticRequest *core,
                               DynamicThread* owner) :
    _static_request(core), _dynamic_thread(owner),
    _state(state_ready)
{
  assert(core != NULL);
  assert(owner != NULL);
	owner->get_requests().push_back(this);
}


DynamicRequest::~DynamicRequest()
{
	for_each(_dynamic_subrequests.begin(), _dynamic_subrequests.end(), ptr_fun(operator delete));
}


bool
DynamicRequest::operator==(const Request& op2) const
{
	assert(dynamic_cast<const DynamicRequest*>(&op2) != NULL);
  return _static_request == dynamic_cast<const DynamicRequest&>(op2)._static_request;
}


vector<SubRequest*>
DynamicRequest::get_subrequests()
{
  return std::vector<SubRequest*>(_dynamic_subrequests.begin(), _dynamic_subrequests.end());
}


vector<DynamicSubRequest*>&
DynamicRequest::get_dynamic_subrequests()
{
  return _dynamic_subrequests;
}

DynamicThread&
DynamicRequest::get_thread()
{
  return *_dynamic_thread;
}

unsigned int
DynamicRequest::get_instant() const
{
  return _static_request->get_instant();
}

Request::state
DynamicRequest::get_state() const
{
  return _state;
}


Request::state
DynamicRequest::set_state(state new_state)
{
	state temp = _state;
	_state = new_state;
	return temp;
}


void
DynamicRequest::serialize(SerializeVisitor& translator) const
{
  translator.from_request(*this);
}


StaticRequest&
DynamicRequest::get_core()
{
	return *_static_request;
}


const StaticRequest&
DynamicRequest::get_core() const
{
	return *_static_request;
}
